// This Lua file deals with the intros for WipeKart Modes

local INTROLENGTH = 9 + (TICRATE * 3)
rawset(_G, "starttime", 6*TICRATE + (3*TICRATE/4))

local function getStringLen(font, str, base)
	local length = string.len(str)
	if length == 0 then return 0 end
	local pxlength = length * base
	for a=1, length
		local letter = string.sub(str, a, a)
		if letter == "I" and font == "WOFNT" then pxlength = $ + 6 - base end
		if (letter == "0" or letter == "2" or letter == "3" or letter == "4"
		or  letter == "5" or letter == "6" or letter == "7" or letter == "8"
		or  letter == "9") and font == "WOFNT" then
			pxlength = $ + 12 - base
		end
		if letter == "1" and font == "WOFNT" then pxlength = $ + 5 - base end
		if letter == " " then pxlength = $ - (base / 2) end
	end
	return pxlength
end

local function introCard(v, x, y, c)
	v.drawFill(0, y+8, x+2, 12, c)
	v.drawFill(-4, y+20, x-2, 4, c)
	v.drawFill(-8, y+24, x-6, 4, c)
	v.drawFill(-12, y+28, x-10, 4, c)
	v.drawFill(-16, y+32, x-14, 4, c)
end

local speeddiff = {
	[0] = "FLASH",
	[1] = "RAPIER",
	[2] = "PHANTOM"
}

local function cupCrop(str)
	local length = string.len(str)
	if length < 3 then return str end
	if string.upper(str) == "MAP HELL" then return "MAP HELL" end
	local result = ""
	local lastspace = false
	for a = 1, length - 2
		local compare = string.sub(str, a, a+2)
		if string.upper(compare) == "CUP" and lastspace then
			return result .. " CUP"
		end
		compare = string.sub($, 1, 1)
		if compare == " " then
			lastspace = true
		elseif lastspace then
			result = $ .. " " .. compare
			lastspace = false
		else
			result = $ .. compare
		end
	end
	for a = length - 1, length
		local compare = string.sub(str, a, a)
		if compare == " " then
			lastspace = true
		elseif lastspace then
			result = $ .. " " .. compare
			lastspace = false
		else
			result = $ .. compare
		end
	end
	return result
end

local function wipe_intro(v, tic, mode)
	//This sorts out the intro. Tic is how long is left in the intro
	
	local t = max(V_50TRANS, V_90TRANS - V_10TRANS*((min(TICRATE, tic))/4))
	if not ((gametype & GT_MATCH) or modeattacking) then v.draw(205, 80, v.cachePatch("WSPEED" .. gamespeed), t|V_SNAPTOTOP) end
	
	local endx
	local leveltitlelength = getStringLen("WOFNT", mapheaderinfo[gamemap].lvlttl, 14)
	endx = min(tic * 16, 160 + (leveltitlelength / 2))
	
	local y = 95
	introCard(v, endx+2, y+2, 31)
	introCard(v, endx, y, 232)
	
	drawWipeFontString(v, "WOFNT", mapheaderinfo[gamemap].lvlttl, endx, y, 14)
	local act = ""
	if mapheaderinfo[gamemap].actnum ~= 0 and mapheaderinfo[gamemap].actnum ~= nil
	and mapheaderinfo[gamemap].actnum ~= "" then
		act = " " .. tostring(mapheaderinfo[gamemap].actnum)
	end
	local zonetext
	if (mapheaderinfo[gamemap].levelflags & LF_NOZONE) then
		zonetext = ""
	elseif mapheaderinfo[gamemap].zonttl ~= ""
		zonetext = mapheaderinfo[gamemap].zonttl .. act
	else
		zonetext = "ZONE" .. act
	end
	local zonetextstart = endx
	if zonetext ~= "" then zonetextstart = drawWipeFontString(v, "WOFNT", zonetext, endx-4, y+20, 14) end
	if mapheaderinfo[gamemap].subttl ~= "Battle Mode" then
		local subtitle = cupCrop(mapheaderinfo[gamemap].subttl)
		v.drawString(zonetextstart - 4, y+24, subtitle, 0, "right")
	end
	
	t = max(V_10TRANS, V_90TRANS - V_10TRANS*((min(TICRATE, tic))/4))
	local modelen = getStringLen("WOFNT", mode, 14)
	drawWipeFontString(v, "WOFNT", mode, 160 + (modelen / 2), y+45, 14, t)
	if not (gametype & GT_MATCH) then
		v.drawString(160, y+65, speeddiff[gamespeed] .. " class, " .. numlaps .. " laps", t, "center")
	end
end

hud.add(function(v, p, c)
	if (leveltime <= INTROLENGTH) then
		local mode
		if (gametype & GT_MATCH) then
			mode = "Battle"
		elseif modeattacking then
			mode = "Time Attack"
		else
			mode = "Race"
		end
		hud.disable("stagetitle")
		local tic = INTROLENGTH - leveltime
		wipe_intro(v, tic, mode)
	end
end)